$fs=0.25;
$fa=2.5;

// Case dimensions
$case_width=46; // Inner width
$case_length=56.2; // Inner length
$case_height=25; // Inner height
$case_thickness=2; // Wall thickness

// Mounting plate
$mount_length=10; // Length of mounting plate
$mount_height=3; // Height of mounting plate
$mount_space=5; // Space of mounting holes from both sides
$mount_diameter=3; // Diameter of mounting hole (M3)

// PCB dimensions
$pcb_width=40; // Circuit board width for centering the connector
$pcb_thickness=1.54; // Circuit board thickness

// PCB holder dimensions
$pcb_holder_height=3; // Height of PCB holders
$pcb_holder_diameter=5; // Outer diameter of pcb holders
$pcb_holder_hole=1.9; // Diameter of PCB mounting holes (M2)

// PCB holder positions
$pcb_holder1_x=2.5;
$pcb_holder1_y=2.5;
$pcb_holder2_x=2.5;
$pcb_holder2_y=53.5;
$pcb_holder3_x=37.5;
$pcb_holder3_y=26;
$pcb_holder4_x=37.5;
$pcb_holder4_y=2.5;

// Connector cutout size
$cutout_width=33;
$cutout_height=16;

// Memory card cutout size
$sdcard_cutout=true;
$sdcard_pos=34.5;
$sdcard_width=15;
$sdcard_length=8;
$sdcard_holder_width=5;
$sdcard_holder_length=1;
$sdcard_holder_height=0.8;

// Lid holder dimensions
$lid_holder_dia=8; // Outer diameter
$lid_holder_thickness=2; // Thickness
$lid_holder_hole=2.4; // Hole size (M2.5)
$lid_holder_edge=3; // Holder position measured from both inner sides

// Dimensions for countersunk head screws (DIN 965 / M2.5*8mm)
$lid_screw_d1=2.5;
$lid_screw_d2=4.7;
$lid_screw_k=1.5;

// Lid text settings
$font_size=4;
$font_depth=0.5;

// Box & cutout
difference() {
    union() {
        cube(size = [$case_width+($case_thickness*2), $case_length+($case_thickness*2), $case_height+$case_thickness], center = false);       
    }
    union() {
        translate([$case_thickness, $case_thickness, $case_thickness]) {
            cube(size = [$case_width, $case_length, $case_height], center = false);
        }
        translate([$case_thickness+(($pcb_width-$cutout_width)/2), 0, $case_thickness+$pcb_holder_height+$pcb_thickness]) {
            cube(size = [$cutout_width, $case_thickness, $cutout_height], center = false);
        }
        if($sdcard_cutout) {
            translate([$case_width+($case_thickness*2)-($sdcard_length+$case_width-$pcb_width),$case_thickness+$sdcard_pos,0]) {
                cube(size = [$sdcard_length+$case_width-$pcb_width, $sdcard_width, $case_thickness+$pcb_holder_height+$pcb_thickness], center = false);
            }
        }
    }
}

// Wall mount
union() {
    translate([0,-$mount_length,0]) {
        difference() {
            cube(size = [$case_width+($case_thickness*2), $mount_length, $mount_height], center = false);
            union() {
                translate([$mount_space,$mount_length/2,0]) {
                    cylinder(h=$mount_height, r=$mount_diameter/2, center=false);
                }
                translate([$case_width+($case_thickness*2)-$mount_space,$mount_length/2,0]) {
                    cylinder(h=$mount_height, r=$mount_diameter/2, center=false);
                }
            }
        }
    }
    translate([0,$case_length+($case_thickness*2),0]) {
        difference() {
            cube(size = [$case_width+($case_thickness*2), $mount_length, $mount_height], center = false);
            union() {
                translate([$mount_space,$mount_length/2,0]) {
                    cylinder(h=$mount_height, r=$mount_diameter/2, center=false);
                }
                translate([$case_width+($case_thickness*2)-$mount_space,$mount_length/2,0]) {
                    cylinder(h=$mount_height, r=$mount_diameter/2, center=false);
                }
            }
        }
    }
}

// PCB mount
difference() {
    union() {
        translate([$pcb_holder1_x+$case_thickness, $pcb_holder1_y+$case_thickness, $case_thickness+($pcb_holder_height/2)]) {
            cylinder(h=$pcb_holder_height, r=$pcb_holder_diameter/2, center=true);  
        }
        translate([$pcb_holder2_x+$case_thickness, $pcb_holder2_y+$case_thickness, $case_thickness+($pcb_holder_height/2)]) {
            cylinder(h=$pcb_holder_height, r=$pcb_holder_diameter/2, center=true);  
        }
        translate([$pcb_holder3_x+$case_thickness, $pcb_holder3_y+$case_thickness, $case_thickness+($pcb_holder_height/2)]) {
            cylinder(h=$pcb_holder_height, r=$pcb_holder_diameter/2, center=true);  
        }
        translate([$pcb_holder4_x+$case_thickness, $pcb_holder4_y+$case_thickness, $case_thickness+($pcb_holder_height/2)]) {
            cylinder(h=$pcb_holder_height, r=$pcb_holder_diameter/2, center=true);  
        }
    }
    union() {
        translate([$pcb_holder1_x+$case_thickness, $pcb_holder1_y+$case_thickness, $case_thickness+($pcb_holder_height/2)]) {
            cylinder(h=$pcb_holder_height, r=$pcb_holder_hole/2, center=true);  
        }
        translate([$pcb_holder2_x+$case_thickness, $pcb_holder2_y+$case_thickness, $case_thickness+($pcb_holder_height/2)]) {
            cylinder(h=$pcb_holder_height, r=$pcb_holder_hole/2, center=true);  
        }
        translate([$pcb_holder3_x+$case_thickness, $pcb_holder3_y+$case_thickness, $case_thickness+($pcb_holder_height/2)]) {
            cylinder(h=$pcb_holder_height, r=$pcb_holder_hole/2, center=true);  
        }
        translate([$pcb_holder4_x+$case_thickness, $pcb_holder4_y+$case_thickness, $case_thickness+($pcb_holder_height/2)]) {
            cylinder(h=$pcb_holder_height, r=$pcb_holder_hole/2, center=true);  
        }
    }
}

// Lid mount
union() {
    translate([$case_thickness,$case_length+$case_thickness-$lid_holder_edge-$lid_holder_dia,$case_height+$case_thickness-$lid_holder_thickness-($case_thickness/2)]) {
       difference() {
            intersection() {
                cube([$lid_holder_dia/2,$lid_holder_dia,$lid_holder_thickness], center=false);
                translate([0,$lid_holder_dia/2,0]) {
                    cylinder(h=$lid_holder_thickness, r=$lid_holder_dia/2, center=false);
                }
            }
            translate([$lid_holder_dia/4,$lid_holder_dia/2,0]) {
                cylinder(h=$lid_holder_thickness, r=$lid_holder_hole/2, center=false);
            }
        }
    }
    translate([$case_width+$case_thickness-($lid_holder_dia/2),$case_thickness+$lid_holder_edge,$case_height+$case_thickness-$lid_holder_thickness-($case_thickness/2)]) {
        difference() {
            intersection() {
                cube([$lid_holder_dia/2,$lid_holder_dia,$lid_holder_thickness], center=false);
                translate([$lid_holder_dia/2,$lid_holder_dia/2,0]) {
                    cylinder(h=$lid_holder_thickness, r=$lid_holder_dia/2, center=false);
                }
            }
            translate([$lid_holder_dia/4,$lid_holder_dia/2,0]) {
                cylinder(h=$lid_holder_thickness, r=$lid_holder_hole/2, center=false);
            }
        }
    }
}

// Lid
translate([$case_width+($case_thickness*2)+5, 0, 0]) {
    difference() {
        union() {
            cube(size = [$case_width+($case_thickness*2), $case_length+($case_thickness*2), $case_thickness], center = false);
            translate([$case_thickness, $case_thickness, $case_thickness]) {
                cube(size = [$case_width, $case_length, $case_thickness/2], center = false);
            }
        }
        union() {
            translate([$case_thickness+($lid_holder_dia/4),$case_thickness+$lid_holder_edge+($lid_holder_dia/2),0]) {
                cylinder(h=$case_thickness*1.5, r=$lid_holder_hole/2, center=false);
                cylinder(h=$lid_screw_k, r1=$lid_screw_d2/2, r2=$lid_screw_d1/2, center=false);
            }
            translate([$case_width-$case_thickness+($lid_holder_dia/4),$case_length+$case_thickness-$lid_holder_edge-$lid_holder_dia+($lid_holder_dia/2),0]) {
                cylinder(h=$case_thickness*1.5, r=$lid_holder_hole/2, center=false);
                cylinder(h=$lid_screw_k, r1=$lid_screw_d2/2, r2=$lid_screw_d1/2, center=false);
            }
            translate([($case_width+($case_thickness*2))/2,($case_length+($case_thickness*2))/2,0]) {
                linear_extrude($font_depth) {
                    mirror([1,0,0]) {
                        text("Pixel Controller", font="Liberation Sans", size=$font_size, halign = "center", valign = "center");
                    }
                }
            }
        }
    }
}

// Memory card lid
if($sdcard_cutout) {
    union() {
        translate([0,$case_length+($case_thickness*2)+$mount_length+5,0]) {
            difference() {
                cube(size = [$sdcard_length+$case_width-$pcb_width, $sdcard_width, $case_thickness+$pcb_holder_height+$pcb_thickness], center = false);
                translate([$case_thickness,0,$case_thickness]) {
                    cube(size = [$sdcard_length+$case_width-$pcb_width, $sdcard_width, $case_thickness+$pcb_holder_height+$pcb_thickness], center = false);
                }  
            }    
            translate([$sdcard_length+$case_width-$pcb_width-($sdcard_holder_length/2), ($sdcard_width-$sdcard_holder_width)/2, $case_thickness]) {
                cube(size = [$sdcard_holder_length, $sdcard_holder_width, $sdcard_holder_height], center = false);
            }
            translate([$case_thickness, ($sdcard_width-$sdcard_holder_width)/2, $case_thickness+$pcb_holder_height+$pcb_thickness-($sdcard_holder_length/2)]) {
                cube(size = [$sdcard_holder_height, $sdcard_holder_width, $sdcard_holder_length], center = false);
            }            
        }
    }
}